@@ -48,6 +48,7 @@ my @CFG = (
 	   # We can't base our check on $], because that's the version of the
 	   # perl that we are running, not the version of the source tree.
 	   "Cross/config.sh-arm-linux",
+	   "Cross/config.sh-arm-linux-n770",
 	   "NetWare/config.wc",
 	   "symbian/config.sh",
 	   "uconfig.sh",
@@ -55,7 +56,6 @@ my @CFG = (
 	   "plan9/config_sh.sample",
 	   "win32/config.gc",
 	   "win32/config.vc",
-	   "win32/config.ce",
 	   "configure.com",
 	   "Porting/config.sh",
 	  );
