@@ -96,6 +96,8 @@ my %skip;
     'cpan/version/t/coretests.pm', # just a test module
     'dist/Attribute-Handlers/demo/MyClass.pm', # it's just demonstration code
     'dist/Exporter/lib/Exporter/Heavy.pm',
+    'dist/Module-CoreList/lib/Module/CoreList.pm',
+    'dist/Module-CoreList/lib/Module/CoreList/Utils.pm',
     'lib/Carp/Heavy.pm',
     'lib/Config.pm',		# no version number but contents will vary
     'win32/FindExt.pm',
