@@ -72,7 +72,7 @@ our @EXPORT_OK = qw(
 	YESSTR
 );
 
-our $VERSION = '0.16';
+our $VERSION = '0.19';
 
 XSLoader::load();
 
@@ -92,7 +92,7 @@ I18N::Langinfo - query locale information
 The langinfo() function queries various locale information that can be
 used to localize output and user interfaces.  It uses the current underlying
 locale, regardless of whether or not it was called from within the scope of
-S<C<use locale>>.  The langinfo() requires
+S<C<use locale>>.  The langinfo() function requires
 one numeric argument that identifies the locale constant to query:
 if no argument is supplied, C<$_> is used.  The numeric constants
 appropriate to be used as arguments are exportable from I18N::Langinfo.
@@ -119,7 +119,13 @@ but under a French locale
 
     dim? [oui/non]
 
-The usually available constants are
+The usually available constants are as follows.
+
+=over 4
+
+=item *
+
+For abbreviated and full length days of the week and months of the year:
 
     ABDAY_1 ABDAY_2 ABDAY_3 ABDAY_4 ABDAY_5 ABDAY_6 ABDAY_7
     ABMON_1 ABMON_2 ABMON_3 ABMON_4 ABMON_5 ABMON_6
@@ -128,41 +134,56 @@ The usually available constants are
     MON_1 MON_2 MON_3 MON_4 MON_5 MON_6
     MON_7 MON_8 MON_9 MON_10 MON_11 MON_12
 
-for abbreviated and full length days of the week and months of the year,
+=item *
+
+For the date-time, date, and time formats used by the strftime() function
+(see L<POSIX>):
 
     D_T_FMT D_FMT T_FMT
 
-for the date-time, date, and time formats used by the strftime() function
-(see L<POSIX>)
+=item *
+
+For the locales for which it makes sense to have ante meridiem and post
+meridiem time formats:
 
     AM_STR PM_STR T_FMT_AMPM
 
-for the locales for which it makes sense to have ante meridiem and post
-meridiem time formats,
+=item *
+
+For the character code set being used (such as "ISO8859-1", "cp850",
+"koi8-r", "sjis", "utf8", etc.), and for the currency string:
 
     CODESET CRNCYSTR
 
-for the character code set being used (such as "ISO8859-1", "cp850",
-"koi8-r", "sjis", "utf8", etc.), for the currency string
+=item *
+
+For an alternate representation of digits, for the
+radix character used between the integer and the fractional part
+of decimal numbers, the group separator string for large-ish floating point
+numbers (yes, the final two are redundant with
+L<POSIX::localeconv()|POSIX/localeconv>):
 
     ALT_DIGITS RADIXCHAR THOUSEP
 
-for an alternate representation of digits, for the
-radix character used between the integer and the fractional part
-of decimal numbers, the group separator string for large-ish floating point
-numbers  (yes, the final two are redundant with POSIX::localeconv())
+=item *
+
+For the affirmative and negative responses and expressions:
 
     YESSTR YESEXPR NOSTR NOEXPR
 
-for the affirmative and negative responses and expressions, and
+=item *
+
+For the eras based on typically some ruler, such as the Japanese Emperor
+(naturally only defined in the appropriate locales):
 
     ERA ERA_D_FMT ERA_D_T_FMT ERA_T_FMT
 
-for the eras based on typically some ruler, such as the Japanese Emperor
-(naturally only defined in the appropriate locales).
+=back
+
+=head2 For systems without C<nl_langinfo>
 
 Starting in Perl 5.28, this module is available even on systems that lack a
-nativeC<nl_langinfo>.  On such systems, it uses various methods to construct
+native C<nl_langinfo>.  On such systems, it uses various methods to construct
 what that function, if present, would return.  But there are potential
 glitches.  These are the items that could be different:
 
@@ -254,7 +275,7 @@ workaround for this; patches welcome: see L<perlapi/switch_to_global_locale>.
 
 L<perllocale>, L<POSIX/localeconv>, L<POSIX/setlocale>, L<nl_langinfo(3)>.
 
-The langinfo() is just a wrapper for the C nl_langinfo() interface.
+The langinfo() function is just a wrapper for the C nl_langinfo() interface.
 
 =head1 AUTHOR
 
